#!/bin/csh -f
#
# Do build of the MITgcm coupled setup
#
set ROOTDIR = ( `pwd`	            )
set LOGDIR  = ( `pwd`	            )
set LOGFILE = ( ${LOGDIR}/build.log )

#
# Check command line options
#
# Arguments
set clean=0
set help=0
set unknown=0
set unrecognised=""
foreach arg ( $* )
 switch ($arg)
   case -clean:
    set clean=1
    set cleanvalue = ( clean )
    breaksw
   case -Clean:
    set clean=1
    set cleanvalue = ( Clean )
    breaksw
   default:
    set help    = 1
    set unknown = 1
    set unrecognised = ( $unrecognised $arg )
 endsw
end

if ( $unknown == 1 ) then
 echo 'Unrecognised options '\"$unrecognised\"
endif
if ( $help == 1 ) then
cat - <<EOF
Usage: $0 '[ options  ]'
 options:
  -help        Show this help message
  -clean       Remove intermediate files
  -Clean       Remove all derived files
EOF
endif

#===================================
# Do the build
#===================================

#
# Build atmospheric code makefile and executable
#
echo "Atmosphere component"
cd $ROOTDIR
cd atm/src/tutorial_examples/aim_5l.LatLon/run
if ( $clean == 0 ) then
echo " 1. Creating/updating makefile"
../../../tools/genmake -mpi -mods=../code      |& tee -a $LOGFILE
#
echo " 2. Checking/rechecking file dependencies"
make depend                                    |& tee -a $LOGFILE
#
echo " 3. Running make"
make                                           |& tee -a $LOGFILE
echo " 4. Atmosphere component build done"
cd $ROOTDIR
cd atm/src/tutorial_examples/aim_5l.LatLon/run
else
make $cleanvalue
endif

#
# Build ocean code makefile and executable
#
echo "Ocean component"
cd $ROOTDIR
cd ocn/src/tutorial_examples/global_ocean.128x64x15/run
if ( $clean == 0 ) then
echo " 1. Creating/updating makefile"
../../../tools/genmake -mpi -mods=../code      |& tee -a $LOGFILE
#
echo " 2. Checking/rechecking file dependencies"
make depend                                    |& tee -a $LOGFILE
#
echo " 3. Running make"
make                                           |& tee -a $LOGFILE
echo " 4. Ocean component build done"
else
make $cleanvalue
endif

#
# Build atmosphere-ocean coupler makefile and executable
#
echo "Atmosphere-ocean coupler component"
cd $ROOTDIR
cd cpl/src/tutorial_examples/aim.5l_LatLon+global_ocean.128x64x15/run
if ( $clean == 0 ) then
echo " 1. Creating/updating makefile"
../../../tools/genmake -mpi -mods=../code      |& tee -a $LOGFILE
#
echo " 2. Checking/rechecking file dependencies"
make depend                                    |& tee -a $LOGFILE
#
echo " 3. Running make"
make                                           |& tee -a $LOGFILE
echo " 4. Atmosphere-ocean coupler component build done"
else
make $cleanvalue
endif

cd $ROOTDIR
echo " "
echo "============================================================="
echo "Build/rebuild done at "`date`" for the following component executables."

echo " o "\"atm/src/tutorial_examples/aim_5l.LatLon/run/mitgcmuv\"
if ( -x atm/src/tutorial_examples/aim_5l.LatLon/run/mitgcmuv ) then
 ls -altr atm/src/tutorial_examples/aim_5l.LatLon/run/mitgcmuv          \
 | awk '{print "    create date "$6,$7,$8}'
else
 echo "    "Executable file not found, check logfile for messages.
endif

echo " o "\"ocn/src/tutorial_examples/global_ocean.128x64x15/run/mitgcmuv\"
if ( -x ocn/src/tutorial_examples/global_ocean.128x64x15/run/mitgcmuv ) then
 ls -altr ocn/src/tutorial_examples/global_ocean.128x64x15/run/mitgcmuv \
 | awk '{print "    create date "$6,$7,$8}'
else
 echo "    "Executable file not found, check logfile for messages.
endif

echo " o "\"cpl/src/tutorial_examples/aim.5l_LatLon+global_ocean.128x64x15/run/mitgcmuv\"
if ( -x cpl/src/tutorial_examples/aim.5l_LatLon+global_ocean.128x64x15/run/mitgcmuv ) then
 ls -altr cpl/src/tutorial_examples/aim.5l_LatLon+global_ocean.128x64x15/run/mitgcmuv \
 | awk '{print "    create date "$6,$7,$8}'
else
 echo "    "Executable file not found, check logfile for messages.
endif

echo File \"$LOGFILE\" contains the log of messages from the build.

