#!/bin/csh
#
# Run JMC code l baseline test (2.8degree coupled, 10 days from 50 year spin up)
#
set ROOTDIR = ( `pwd` )

#
# Install executables
cd $ROOTDIR
\cp atm/src/tutorial_examples/aim_5l.LatLon/run/mitgcmuv                        atm/run
\cp ocn/src/tutorial_examples/global_ocean.128x64x15/run/mitgcmuv               ocn/run
\cp cpl/src/tutorial_examples/aim.5l_LatLon+global_ocean.128x64x15/run/mitgcmuv cpl/run

# Link inputs into executable directories
cd $ROOTDIR
cd atm/run ;ln -fs ../input/* .
cd $ROOTDIR
cd ocn/run ;ln -fs ../input/* .

# Submit a batch job and wait for completion
cd $ROOTDIR
setenv ROOTDIR $ROOTDIR
bsub -q esto_big -n 17 -o stdout -e stderr -c 00:10 -I < tj-driver

# Process timings and validation numbers
grep -i theta_mean atm/run/STDOUT.0000 > STDOUT.0000.cmp
cmp STDOUT.0000.cmp STDOUT.0000.ref >>& /dev/null
set rc = $status
if ( $rc == 0 ) then
 echo 'Validation PASSED' >& output.txt
else
 echo 'Validation error'
 exit
endif
set wtime = ( `cat atm/run/STDOUT.0000 | awk '/FORWARD_STEP/,/EXTERNAL_FIELDS_LOAD/{print}' | grep Wall | awk -F: '{print $2}'` )
echo 'Wall clock time to solution for a ten day run is' $wtime' seconds on 17 processors' >>& output.txt
set yearsperday = ( ` echo $wtime | awk '{printf("%.1f",86400/$1/36)}' ` )
echo 'This configuration will simulate '$yearsperday' years per one day of compute time'  >>& output.txt
cat output.txt
echo 'ALL DONE'
